"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildUrl = exports.normalizeRecord = exports.prepareProperties = exports.suggestPropertyType = exports.getValueParameters = exports.arrayIsConsistent = exports.isUUID = exports.createSearchParams = exports.pickRecordId = exports.pickTransactionId = exports.pickTransaction = exports.isTransaction = exports.buildTransactionHeader = void 0;
const constants_js_1 = require("../common/constants.js");
const utils_js_1 = require("../common/utils.js");
const sdk_1 = require("../sdk");
const transaction_js_1 = require("../sdk/transaction.js");
const constants_js_2 = require("./constants.js");
const buildTransactionHeader = (txId) => txId ?
    {
        ['x-transaction-id']: txId
    }
    : undefined;
exports.buildTransactionHeader = buildTransactionHeader;
const isTransaction = (input) => (0, utils_js_1.isString)(input) || input instanceof transaction_js_1.Transaction;
exports.isTransaction = isTransaction;
const pickTransaction = (input) => ((0, exports.isTransaction)(input) ? input : undefined);
exports.pickTransaction = pickTransaction;
const pickTransactionId = (input) => (0, exports.isTransaction)(input) ?
    input instanceof transaction_js_1.Transaction ?
        input.id
        : input
    : undefined;
exports.pickTransactionId = pickTransactionId;
const pickRecordId = (input) => {
    if ((0, utils_js_1.isString)(input)) {
        return input;
    }
    else if (input instanceof sdk_1.DBRecordInstance && input.data) {
        return input.data.__id;
    }
    else if ('__id' in input) {
        return input.__id;
    }
    return undefined;
};
exports.pickRecordId = pickRecordId;
const createSearchParams = (labelOrSearchParams, searchParamsOrTransaction) => {
    var _a, _b, _c;
    const isFirstArgString = (0, utils_js_1.isString)(labelOrSearchParams);
    const isFirstArgUUID = (0, exports.isUUID)(labelOrSearchParams);
    const isSecondArgTransaction = (0, exports.isTransaction)(searchParamsOrTransaction);
    const isEmptySearchParams = isSecondArgTransaction || !(0, utils_js_1.isObject)(searchParamsOrTransaction);
    if (isFirstArgString) {
        const baseParams = isFirstArgUUID ?
            { id: labelOrSearchParams }
            : { searchParams: { labels: [labelOrSearchParams] } };
        return isEmptySearchParams ? Object.assign(Object.assign({}, baseParams), { searchParams: Object.assign({}, baseParams.searchParams) }) : Object.assign(Object.assign({}, baseParams), { searchParams: Object.assign(Object.assign({}, searchParamsOrTransaction), { labels: [
                    ...((_b = (_a = baseParams.searchParams) === null || _a === void 0 ? void 0 : _a.labels) !== null && _b !== void 0 ? _b : []),
                    ...((_c = searchParamsOrTransaction.labels) !== null && _c !== void 0 ? _c : [])
                ] }) });
    }
    else {
        return { searchParams: labelOrSearchParams !== null && labelOrSearchParams !== void 0 ? labelOrSearchParams : {} };
    }
};
exports.createSearchParams = createSearchParams;
const isUUID = (value) => {
    const regex = /^[0-9a-fA-F]{8}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{12}$/;
    return regex.test(value);
};
exports.isUUID = isUUID;
const arrayIsConsistent = (arr) => arr.every((item) => typeof item === typeof arr[0]);
exports.arrayIsConsistent = arrayIsConsistent;
const getValueParameters = (value) => {
    if (Array.isArray(value)) {
        return {
            isEmptyArray: value.length === 0,
            isEmptyStringsArray: value.every((v) => v === ''),
            isInconsistentArray: !(0, exports.arrayIsConsistent)(value)
        };
    }
    else {
        return { isEmptyString: value === '' };
    }
};
exports.getValueParameters = getValueParameters;
const suggestPropertyType = (value) => {
    if (typeof value === constants_js_1.PROPERTY_TYPE_STRING) {
        return constants_js_1.ISO_8601_FULL.test(value) ? constants_js_1.PROPERTY_TYPE_DATETIME : constants_js_1.PROPERTY_TYPE_STRING;
    }
    else if (typeof value === constants_js_1.PROPERTY_TYPE_NUMBER) {
        return constants_js_1.PROPERTY_TYPE_NUMBER;
    }
    else if (typeof value === constants_js_1.PROPERTY_TYPE_BOOLEAN) {
        return constants_js_1.PROPERTY_TYPE_BOOLEAN;
    }
    else if (value === null) {
        return constants_js_1.PROPERTY_TYPE_NULL;
    }
    else {
        return constants_js_1.PROPERTY_TYPE_STRING;
    }
};
exports.suggestPropertyType = suggestPropertyType;
const processArrayValue = (value, suggestTypes) => {
    const { isEmptyArray, isInconsistentArray } = (0, exports.getValueParameters)(value);
    if (isEmptyArray) {
        return { type: constants_js_1.PROPERTY_TYPE_STRING, value: [] };
    }
    if (isInconsistentArray || !suggestTypes) {
        return { type: constants_js_1.PROPERTY_TYPE_STRING, value: value.map(String) };
    }
    return { type: (0, exports.suggestPropertyType)(value[0]), value };
};
const processNonArrayValue = (value, suggestTypes) => {
    if (!suggestTypes) {
        return { type: constants_js_1.PROPERTY_TYPE_STRING, value: String(value) };
    }
    const type = (0, exports.suggestPropertyType)(value);
    return { type, value: type === constants_js_1.PROPERTY_TYPE_NULL ? null : value };
};
const prepareProperties = (data, options = { suggestTypes: true }) => Object.entries(data).map(([name, value]) => {
    const { type, value: processedValue } = (0, utils_js_1.isArray)(value) ?
        processArrayValue(value, options.suggestTypes)
        : processNonArrayValue(value, options.suggestTypes);
    return { name, type, value: processedValue };
});
exports.prepareProperties = prepareProperties;
const normalizeRecord = ({ label, options = { suggestTypes: true }, payload }) => ({
    label,
    properties: (0, exports.prepareProperties)(payload, options)
});
exports.normalizeRecord = normalizeRecord;
const buildUrl = (props) => {
    let protocol = constants_js_2.DEFAULT_PROTOCOL;
    let host = constants_js_2.DEFAULT_HOST;
    let port = constants_js_2.DEFAULT_PORT;
    let basePath = constants_js_2.DEFAULT_BASE_PATH;
    if ('url' in props) {
        const url = new URL(props.url);
        protocol = url.protocol.replace(':', '');
        host = url.hostname;
        port = parseInt(url.port ||
            (protocol === 'http' ? '80'
                : protocol === 'https' ? '443'
                    : ''));
    }
    if ('host' in props && 'port' in props && 'protocol' in props) {
        protocol = props.protocol.replace(':', '');
        host = props.host;
        port = props.port;
    }
    // Ensure the basePath starts with a '/'
    if (basePath && !basePath.startsWith('/')) {
        basePath = '/' + basePath;
    }
    // If the port is the default for the protocol (80 for http, 443 for https), it can be omitted
    let portString = '';
    if (!((protocol === 'http' && port === 80) || (protocol === 'https' && port === 443))) {
        portString = ':' + port;
    }
    return `${protocol}://${host}${portString}${basePath}`;
};
exports.buildUrl = buildUrl;
