import type { AnyObject, PropertyType, PropertyValue, PropertyWithValue, SearchQuery, Schema, FlattenTypes, InferSchemaTypesRead, InferSchemaTypesWrite, MaybeArray, OptionalKeysRead, RequiredKeysRead } from '../types/index.js';
import type { Transaction } from './transaction.js';
import { RestApiProxy } from '../api/rest-api-proxy.js';
type DBRecordInternalProps<S extends Schema = Schema> = {
    readonly __id: string;
    readonly __label: string;
    readonly __proptypes?: FlattenTypes<{
        [Key in RequiredKeysRead<S>]: S[Key]['type'];
    } & {
        [Key in OptionalKeysRead<S>]?: S[Key]['type'];
    }>;
};
export type RecordProps<S extends Schema = Schema> = S extends S ? InferSchemaTypesRead<S> : {
    [K in keyof S]?: S[K];
};
export type DBRecord<S extends Schema = Schema> = FlattenTypes<DBRecordInternalProps<S> & FlattenTypes<RecordProps<S>>>;
export type DBRecordTarget = DBRecord<any> | DBRecordInstance<any> | string;
export type RelationTarget = DBRecordsArrayInstance<any> | MaybeArray<DBRecord<any>> | MaybeArray<DBRecordInstance<any>> | MaybeArray<string>;
export type Relation = {
    sourceId: string;
    sourceLabel: string;
    targetId: string;
    targetLabel: string;
    type: string;
};
export type RelationDirection = 'in' | 'out';
export type RelationOptions = {
    direction?: RelationDirection;
    type?: string;
};
export type RelationDetachOptions = {
    direction?: RelationDirection;
    typeOrTypes?: string | string[];
};
export declare class DBRecordsBatchDraft {
    label: string;
    options?: {
        returnResult?: boolean;
        suggestTypes?: boolean;
    };
    payload: MaybeArray<AnyObject>;
    constructor({ label, options, payload }: {
        label: string;
        options?: {
            returnResult?: boolean;
            suggestTypes?: boolean;
        };
        payload: AnyObject;
    });
    toJson(): {
        label: string;
        options: {
            returnResult?: boolean;
            suggestTypes?: boolean;
        } | undefined;
        payload: MaybeArray<AnyObject>;
    };
}
export declare class DBRecordDraft {
    label: string;
    properties?: Array<{
        metadata?: string;
        name: string;
        type: PropertyType;
        value: PropertyValue;
        valueSeparator?: string;
    }>;
    constructor({ label, properties }: {
        label: string;
        properties?: Array<PropertyWithValue & {
            metadata?: string;
            valueSeparator?: string;
        }>;
        relation?: RelationOptions;
    });
    toJson(): {
        label: string;
        properties: {
            metadata?: string;
            name: string;
            type: PropertyType;
            value: PropertyValue;
            valueSeparator?: string;
        }[] | undefined;
    };
}
export declare class DBRecordInstance<S extends Schema = Schema> extends RestApiProxy {
    data?: DBRecord<S>;
    searchParams?: SearchQuery<S>;
    constructor(data?: DBRecord<S>, searchParams?: SearchQuery<S>);
    __date(): Date;
    __timestamp(): number;
    delete(transaction?: Transaction | string): Promise<import("../api/types.js").ApiResponse<{
        message: string;
    }, Record<string, any>> | undefined>;
    update<S extends Schema = Schema>(partialData: DBRecordDraft | Partial<InferSchemaTypesWrite<S>>, transaction?: Transaction | string): Promise<DBRecordInstance<S> | undefined>;
    set<S extends Schema = Schema>(data: DBRecordDraft | InferSchemaTypesWrite<S>, transaction?: Transaction | string): Promise<DBRecordInstance<S> | undefined>;
    attach(target: RelationTarget, options?: RelationOptions, transaction?: Transaction | string): Promise<import("../api/types.js").ApiResponse<{
        message: string;
    }, Record<string, any>> | undefined>;
    detach(target: RelationTarget, options?: RelationDetachOptions, transaction?: Transaction | string): Promise<import("../api/types.js").ApiResponse<{
        message: string;
    }, Record<string, any>> | undefined>;
}
export declare class DBRecordsArrayInstance<S extends Schema = Schema> extends RestApiProxy {
    data?: DBRecord<S>[];
    total: number | undefined;
    searchParams?: SearchQuery<S>;
    constructor(data?: DBRecord<S>[], total?: number, searchParams?: SearchQuery<S>);
}
export {};
